# Repository path:    $HeadURL: https://svn.dmhoutribdijk.nl/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorStatus.py $
# Last committed:     $Revision: 130491 $; $Author: mvhoek $; $Date: 2019-05-29 12:18:27 +0200 (Wed, 29 May 2019) $
#
from Utils import Utils
from DmsLogger import DmsLogger
from ReadFewsPy import ReadFewsPy
import pandas as pd
import json
import numpy as np
import sys
import getopt
import re
import datetime
import glob
import os


def timestamp_of_last_file_in_dir(sfolder, sensor_loc, dir_file_ts):
    list_of_files = glob.glob(os.path.join(sfolder, "*.dat"))
    if list_of_files:
        latest_file = max(list_of_files, key=os.path.getctime)
        latest_file_ts = datetime.datetime.utcfromtimestamp(os.path.getctime(latest_file))
        dir_file_ts.append(
            {
                "key_importbackup": sensor_loc,
                "file_importbackup": latest_file,
                "ts_importbackup": latest_file_ts,
            }
        )
    return dir_file_ts


def subfolders(folder):
    subfolders = [f.path for f in os.scandir(folder) if f.is_dir()]
    return subfolders


def timestamp_importbackup(folder):
    dir_file_ts = []
    sensor_locations = subfolders(folder)
    for sensor_location in sensor_locations:
        key_sensor_loc = os.path.basename(sensor_location)
        timestamp_of_last_file_in_dir(sensor_location, key_sensor_loc, dir_file_ts)

        sensor_sublocations = subfolders(sensor_location)
        for sensor_sublocation in sensor_sublocations:
            key_sensor_subloc = "{}_{}".format(
                key_sensor_loc, os.path.basename(sensor_sublocation)
            )
            timestamp_of_last_file_in_dir(
                sensor_sublocation, key_sensor_subloc, dir_file_ts
            )

    df_file_ts = pd.DataFrame().from_dict(dir_file_ts)
    return df_file_ts


def timestamp_status_data(folder_ftpstatus_status, folder_ftpstatus_data):

    list_of_status_files = glob.glob(
        os.path.join(folder_ftpstatus_status, "*STA_*.json")
    )
    list_of_status_files = [list_of_status_files[0], list_of_status_files[-1]]

    df_status_data_sensor_loc_ts = pd.DataFrame()
    for status_file in list_of_status_files:
        status_loc = os.path.basename(status_file).split("_")

        if len(status_loc) == 4:
            key_status_loc = "{}".format(status_loc[2])
        elif len(status_loc) == 5:
            key_status_loc = "{}_{}".format(status_loc[2], status_loc[3])

        # get creation time status file
        status_file_ts = datetime.datetime.utcfromtimestamp(
            os.path.getctime(status_file)
        )

        # get path of status data file
        df_status_status = pd.read_json(status_file, typ="series", orient="values")
        status_file_ts = pd.to_datetime(df_status_status["ts"])
        status_data_file = df_status_status["fn"]
        path_status_data_file = os.path.join(
            folder_ftpstatus_data, key_status_loc.replace("_", "\\"), status_data_file
        )

        # read status data file
        df_status_data = pd.read_csv(path_status_data_file, skiprows=[0, 2, 3])

        sensors = [
            "ETRO_Status",
            "ISA_Status",
            "YSI_Status",
            "AQD_Status",
            "VEC_Status",
            "RDI_Status",
        ]
        s_status_sensors = df_status_data[sensors].isin([-1]).any()
        s_status_sensors.index = [sensor.split("_")[0] for sensor in sensors]

        df_status_sensors = s_status_sensors.to_frame()
        df_status_sensors.reset_index(inplace=True)
        df_status_sensors.columns = ["data_sensor", "data_available"]

        df_status_sensors.loc[:, "file_ftpstatus_data"] = path_status_data_file
        df_status_sensors.loc[:, "file_ftpstatus"] = status_file
        df_status_sensors.loc[:, "key_ftpstatus"] = key_status_loc
        df_status_sensors.loc[:, "ts_ftpstatus"] = status_file_ts

        df_status_data_sensor_loc_ts = df_status_data_sensor_loc_ts.append(
            df_status_sensors
        )

    return df_status_data_sensor_loc_ts


def error_message(df):

    # Check if timedelta is more than 2 hours and data is available
    msg_filter = [
        all(tup)
        for tup in zip(df.data_available == True, df.ts_delta > pd.Timedelta(hours=2))
    ]
    df_msg = df

    # add columns for prtg
    df_msg.loc[:, "channel"] = df_msg["key_ftpstatus"] + " - " + df_msg["data_sensor"]
    df_msg.loc[:, "value_error"] = 0
    df_msg.loc[msg_filter, "value_error"] = 2
    df_msg.loc[:, "limitmaxerror"] = 1
    df_msg.loc[:, "limitmode"] = 1
    df_msg.loc[
        :, "limiterrormsg"
    ] = "Actieve sensor data is succesvol geimporteerd in het DMS"
    df_msg.loc[msg_filter, "limiterrormsg"] = [
        "Melding: File {} is op ftp aanwezig maar niet geimporteerd in het DMS".format(
            time_of_error
        )
        for time_of_error in df.loc[msg_filter, "file_ftpstatus_data"]
    ]

    # filter columns to output
    df_msg_out = df_msg[
        ["channel", "value_error", "limitmaxerror", "limitmode", "limiterrormsg"]
    ]
    df_msg_out.columns = [
        "channel",
        "value",
        "limitmaxerror",
        "limitmode",
        "limiterrormsg",
    ]

    return df_msg_out


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorStatusImportBackup.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorStatusImportBackup.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_monitor_status_import_backup_settings(settings_file)
    except FileNotFoundError:
        exit(1)

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run_MonitorStatusImportBackup")

    folder_importbackup = settings["folder_importbackup"]
    folder_ftpstatus_status = settings["folder_ftpstatus_status"]
    folder_ftpstatus_data = settings["folder_ftpstatus_data"]
    file_monitor_out = settings["file_monitor_out"]

    # get dataframe with timestamps status files and status data of sensor-location combinations
    # get dataframe with timestamps of most recent files in FEWS ImportBackup folder
    # merge dataframes
    # compute timedelta
    # create error message
    df_ftpstatus_data = timestamp_status_data(
        folder_ftpstatus_status, folder_ftpstatus_data
    )
    df_importbackup = timestamp_importbackup(folder_importbackup)
    df_ftp_backup = df_ftpstatus_data.merge(
        df_importbackup, how='inner', left_on="key_ftpstatus", right_on="key_importbackup"
    )
    df_ftp_backup["ts_delta"] = (
        df_ftp_backup["ts_ftpstatus"] - df_ftp_backup["ts_importbackup"]
    )
    df_msg_out = error_message(df_ftp_backup)

    # save to file
    error_out = {"prtg": {"result": df_msg_out.to_dict(orient="records")}}
    with open(file_monitor_out, "w") as f:
        print(json.dumps(error_out), file=f)
    logger.info("Succesfully Finished - MonitorStatusImportBackup")


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
